
package spirograph;

import bsh.Interpreter;
import bsh.EvalError;

import java.util.Hashtable;
import java.util.StringTokenizer;

/** Implements and accelerator that uses bean shell to interpret student code.
 * See <a href="http://www.beanshell.org/">www.beanshell.org</a>
 * for bean shell details.
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology.<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering.</p>
 */
public class BshAccel implements Accelerator {

  private Interpreter i;
  
  /** Create a new Accelerator that uses the bean shell (bsh)
   * interpreter to evaluate code, rather than the native Java
   * compiler.
   * @param fieldCode The student code defining the fields (variables) used in the body code.
   * @param bodyCode The student code to evaluate
   */
  public BshAccel(String fieldCode, String bodyCode)
  {
      String code, mangledCode;
    
      // bsh has some strange ideas about closure semantics. In
      // particular, a variable on the left hand side of an assignment
      // always assigns to a variable in local scope, even if it means
      // creating a new variable in local scope that shadows one in the
      // enclosing scope.  Thus, using normal Java-style syntax,
      // variables in enclosing scopes are read-only.
      //
      // The escape is that "super.foo" will explicitly refer to the
      // variable named foo in the enclosing scope. We don't want to
      // expose this to the students, because it is different from real
      // Java syntax. Therefore, we first parse the "fields" text for a
      // list of variables, and prefix all uses of those variables in
      // the body of the code with the string "super.".
      //
      // If the body declares a variable that would shadow the field,
      // uses of that will also be prefixed, so the student won't see
      // the usual shadowing semantics. This shouldn't be a problem.

      mangledCode = ScopeMangle(fieldCode, bodyCode);

      code = "wrapper()" + 
             "{" +
 	         fieldCode + 
 
	         "foo(double pos, double vel, double otherPos, " +
	         "double otherVel, double maxPos)" + 
	         "{" +
	              mangledCode +
	             "return 0;" + // Ensure some value is returned
	         "}" + 

	         "return this;" + // wacky bsh return-closure syntax
	     "}";

      i = new Interpreter();
      try 
      {
	  i.eval(code);
	  i.eval("wrobj = wrapper();");
      } 
      catch (EvalError e) 
      {
	  System.out.println("Error in student code: " + e);
      }
	    
  }
  
  // method act documented in Accelerator

  public double act(double pos, double vel,
		    double otherPos, double otherVel, double maxPos)
  {
    double ret = 0.0;

    try 
    {
	i.eval("retval = (double) wrobj.foo( " + 
	       pos + ", " + 
	       vel + ", " + 
	       otherPos + ", " + 
	       otherVel + ", " + 
	       maxPos + ") ");
	i.eval("if (retval == void) { retval = 0; }");
	ret = ((Double)i.getVariable("retval")).doubleValue();
    }
    catch (EvalError e) 
    {
	System.out.println("Eval error (internal): " + e);
    }

    return ret;
  }

    String ScopeMangle(String fields, String body) 
    {
	StringTokenizer st;
	Hashtable fieldNames;
	boolean haveType = false;
        boolean inQuote = false;
        boolean inDoubQuote = false;
        boolean escapeNext = false;
	String token, newBody;

	// The grammar of fields we recognize is:
	// "type foo[,bar]*;"
	st = new StringTokenizer(fields, " \t\n\r,;=\"\\'", true);
	fieldNames = new Hashtable();

	// Collect field names.
	haveType = false;
	while (st.hasMoreTokens()) 
	{
	    token = st.nextToken();
            
            if (escapeNext) {
                escapeNext = false;
                continue;
            }
            
            if (token.equals("\"")) {
                if (!inDoubQuote) {
                    inDoubQuote = true;
                    continue;
                } else {
                    inDoubQuote = false;
                    continue;
                }
            }
            
            if (token.equals("'")) {
                if (!inQuote) {
                    inQuote = true;
                    continue;
                } else {
                    inQuote = false;
                    continue;
                }
            }
          
            if (token.equals("\\")) {
                escapeNext = true;
                continue;
            } 
            
            
            if (inQuote || inDoubQuote) {
                continue;
            }
                
            if (token.equals(" ") || 
		token.equals("\t") || 
		token.equals("\n") || 
		token.equals("\r") || 
		token.equals("{") || 
		token.equals("}") || 
		token.equals("[") || 
		token.equals("]") || 
		token.equals(",") || 
		token.equals("=")) {
		; // do nothing
            } else if (Character.isDigit(token.charAt(0))) {
		; // do nothing
            } else if (token.equals(";")) {
		haveType = false;
            } else if(haveType == false) {
		haveType = true;
            } else if (!SpiroUtils.isReservedWord(token)) { // A field name 
		fieldNames.put(token, "field");
            }
	}

	// Mangle the body.
	st = new StringTokenizer(body,
				 " \t\n\r`~!@#%^&*()-=+[]{}\\|;:'\"<>,./?",
				 true);

	newBody = "";
	while (st.hasMoreTokens())
	{
	    token = st.nextToken();

	    // Don't bother figuring out what kind of a thing the
	    // token is. If it's in our list of fields, it's fair
	    // game. 

	    if (fieldNames.containsKey(token))
		token = "super." + token;

	    newBody += token;
	}

	return newBody;
    }
}

/*
 * $Log: BshAccel.java,v $
 * Revision 1.8  2003/01/14 19:41:35  gus
 * Fixed typo and javadoc error
 *
 * Revision 1.7  2003/01/13 20:02:48  gus
 * Remove useless string field that makes the code harder to read.
 *
 * Revision 1.6  2003/01/13 19:59:25  gus
 * Remove entirely unneccessary argument from BshAccel constructor
 *
 * Revision 1.5  2003/01/10 22:19:18  gus
 * Complete and update the javadocs
 *
 * Revision 1.4  2003/01/10 17:57:49  gus
 * Handle quoted string literals and charcter literals
 * Also don't use _ and $ in the delimiters for the body; they are valid chars in identifiers
 *
 */
