/*
 * RecipieModel.java
 *
 * Created on December 10, 2002, 5:17 PM
 */

package spirograph;

import java.io.Serializable;

import java.util.Vector;

import javax.swing.event.ChangeEvent;

import spirograph.RecipieListener;

/** 
 * Models the student code input, or "Recipie". A recipie consists of two
 * strings. The first string may contain any number of variable declarations and
 * defintions. The second string esentially contains a method body which is
 * interpreted to control the movement of a dot on the x or y axis (or both).
 * Classes which implement the {@link RecipieListener} interface may register
 * be notified of changes in either the field text or the statement (method
 * body) text. An always empty static instance of this class is maintained as
 * the "null Recipie". Any recipie variable can point to this static instance
 * and be assured, both that it exists and that it will never have any code
 * or fields text.
 *
 * <p>Copyright &copy; 2003 Franklin W. Olin College of Engineering.</p>
 *
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: Recipie.java,v 1.9 2004/02/09 20:55:03 gus Exp $
 * @see RecipieView
 */
public class Recipie implements Serializable, CodeSource {

    private static int instances = 0;
    private int idNum;
    
    /**
     * A recipie that represents a no data condition.
     * It should always have id 0, and all fields should be empty or zero at
     * all times. Attempts to modify this object should result in an
     * <code>UnsupportedOperationException</code>
     *
     */
    public static final Recipie nullRecipie = new Recipie();
    
    private ChangeEvent ce = new ChangeEvent(this);
    private Vector listeners = new Vector();
    private String fields = "";
    private String code = "";
     
    private boolean compiled = true;
    
    
    /** Creates a new instance of RecipieModel
     */
    public Recipie() {
        idNum = instances;
        instances++;
    }
    
    /**
     * Ask this recipie if it has been compiled since it's last modification.
     *
     * @return True if no modifications have been made since the last
     *           <code>setCompiled(true)</code> call
     */    
    public boolean isCompiled() {
        return compiled;
    }
    
    /** 
     * Get a unique identifier for this object.
     *
     * @return An identfying integer that is unique to any given instance.
     */    
    public int getIdNum() {
        return idNum;
    }

    /**
     * Get the student code that represents a method body.
     *
     * @return The student method body code
     */    
    public String getCode() {
        return code;
    }
    
    /**
     * Get the student code that represents variable declarations and definitions.
     *
     * @return The student variable declarations and defintions
     */    
    public String getFields() {
        return fields;
    }
    
    /**
     * Mark this code as having been compiled since it's last edit, or edited
     * since it's last compile.
     *
     * @param isComp <code>true</code> if the code has been compiled, <code>false</code> if it has been modified
     */    
    public void setCompiled(boolean isComp) {
        compiled = isComp;
        updateListeners();
    }
    
    /** Store new student code.
     * @param statements The student code to store.
     * @throws UnsupportedOperationException if invoked on {@link Recipie#nullRecipie}
     */    
    public void setCode(String statements) throws UnsupportedOperationException {
        if (this == nullRecipie) {
            throw new UnsupportedOperationException("Cannot modify "+
                                                    "the \"null recipie!\"");
        }
        code = statements;
        compiled = false;
        updateListeners();
    }
    
    /** Store new student fields.
     * @param fields The variable declarations and definitions to store.
     * @throws UnsupportedOperationException if invoked on {@link Recipie#nullRecipie}
     */    
    public void setFields(String fields) throws UnsupportedOperationException {
        if (this == nullRecipie) {
            throw new UnsupportedOperationException("Cannot modify "+
                                                    "the \"null recipie!\"");
        }
        fields = fields;
        compiled = false;
        updateListeners();
    }

    /** Register an object for notification of changes to this recipie.
     * @param rl The object to register
     */    
    public void addRecipieListener(RecipieListener rl) {
        listeners.add(rl);        
    }
    
    /** Unregister an object that was listening to this recipie.
     * @param rl The object that wishes to be unregistered.
     */    
    public void removeRecipieListener(RecipieListener rl) {
        listeners.remove(rl);
    }
    
    /**
     * Manually ensure that all listners are updated.
     *
     */    
    public void fireRecipieChanged() {
        updateListeners();
    }
    
    private void updateListeners() {
        for (int i=0; i<listeners.size(); i++){
            ((RecipieListener) listeners.get(i)).recipieUpdated(ce);
        }
    }
}

/*
 * $Log: Recipie.java,v $
 * Revision 1.9  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.8  2003/01/17 23:57:17  gus
 * see tag added
 *
 * Revision 1.7  2003/01/15 17:36:10  gus
 * adding log keywords to files that don't have them
 *
 */