/*
 * SpiroUtils.java 
 * Part of the Spirograph problem set
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's Computers
 * and Cognition Group. For more information, see 
 * <a href="http://www.cognition.olin.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@olin.edu>.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Copyright (C) 2002-2003 Franklin W. Olin College of Engineering.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.io.*;

/**
 * This class contained a bunch of Utilities that were specific to this
 * problem set, but only one of them turned out to be neccesary.
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering
 *
 * @author  Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author  Henry Wong, henryw@mit.edu
 * @author  Patrick G. Heck, gus.heck@olin.edu
 * @version  $Id: SpiroUtils.java,v 1.6 2004/02/09 20:55:03 gus Exp $
 * 
 * @see Accelerator
 */
public abstract class SpiroUtils {
    
    /**
     * The reserved words for java 1.4. Altough the student's shouldn't ever
     * use assert in this lab, I have included it for completeness. If you are
     * using an SDK earlier than 1.4 and a student picks a variable named
     * assert, this will cause problems, but this should be acceptably rare.
     */
    public static final String[] RESERVED_WORDS =
    { "abstract",       "assert",       "boolean",      "break",
      "byte",           "case",          "catch",        "char",
      "class",          "const",        "continue",     "default",
      "do",             "double",       "else",         "extends",
      "false",          "final",        "finally",      "float",
      "for",            "goto",         "if",           "implements",
      "import",         "instanceof",   "int",          "interface",
      "long",           "native",       "new",          "null",
      "package",        "private",      "protected",    "public",
      "return",         "short",        "static",       "stictfp",
      "super",          "switch",       "synchronized", "this",
      "throw",          "throws",       "transient",    "true",
      "try",            "void",         "volatile",     "while" };
    
    /**
     * Check to see if a string contains a java reserved word.
     * 
     * @param someCode The code or other string to check for reserved words
     * @return true if a reserved word is detected, false otherwise
     */

    public static boolean isReservedWord(String someCode) {
        for (int i=0; i<RESERVED_WORDS.length; i++) {
            if (someCode.equals(RESERVED_WORDS[i])) {
                return true;
            }
        }
        return false;
    }
    
    /** This method reads in a class file, creates an Object from that
     * class and casts it to an Accelerator. <p>
     *
     * Class.forName(String) caches names, if I ask it to reread
     * a file that the user has changed it will return the old
     * version of that file. Therefore, I use some system calls
     * to copy the file to a new name, read it in and erase it. <p>
     *
     * This method is only used to load the classes, if any, specified
     * as command line arguments when the program is started.  It is <em>not</em>
     * used to load the classes that are compiled during the execution
     * of the program.  To do this, a cs101.util.MultiClassLoader is used.
     *
     * @return The loaded class if it implements Accelerator, null otherwise.
     * @param className The string specifying the name of the class to load
     */ 
    public static Accelerator createAccel(String className) {
        Object temp = null;
        Class c = null;
        
        // read in temporary file
        try {
            //      MultiClassLoader loader = new FileClassLoader("");
            //loader.monitorOn = true;
            
            //      c = loader.forceLoadClass(className);
            
            // We do not need to use a MultiClassLoader here.  Class.forName works
            // fine, since it is the first time we are loading the class.  Only
            // use the MultiClassLoader when loading a class that has changed.
            c = Class.forName(className);
            System.out.println( "Loaded class " + c.getName() );
            
            temp = c.newInstance();
            
        } catch (Exception e) {
            System.out.println (e);
            System.out.println ("Class "+ className + " not found.");
            return(null);
        }
        
        if (!(temp instanceof Accelerator)) {
            System.out.println ("Class doesn't implement Accelerator");
            return(null);
        }
        
        return (Accelerator)temp;
    }
    
    /** The distance between any two points in 2D space.
     *
     * @param ax first x coordinate
     * @param ay first y coordinate
     * @param bx second x coordinate
     * @param by second y coordinate
     * @return the distance between the (ax,ay) and (bx,by)
     */
    public static double dist(double ax, double ay, double bx, double by) {
        return Math.sqrt(Math.pow(ax-bx,2)+Math.pow(ay-by,2));
    }

    /** Calculates whether or not a given position is inside a
     * given ellipse. The elipse is assumed to be centered on
     * 0,0 with major and minor axes coinciding with
     * the x and y coordinate axes.
     *
     * @param x The x coordinate of the point being tested
     * @param y The y coordinate of the point being tested
     * @param a The major or minor axis of the elipse in the x direction
     * @param b The major or minor axis of the elipse in the y direction
     * @return true if the point is in the elipse false otherwise
     */
    
    public static boolean inEllipse(double x, double y, double a, double b) {
	return (Math.pow(x/a,2)+Math.pow(y/b,2) < 1);
    }

}

/*
 * $Log: SpiroUtils.java,v $
 * Revision 1.6  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.5  2003/01/09 21:29:31  gus
 * renamed hasReservedWord to isReserved word and changed functionality
 * corespondingly. Also cleaned up docs and indentation, and copyright in SpiroUtils
 *
 * Revision 1.4  2003/01/09 20:47:53  gus
 * Added a method to check for java keywords, and completed the missing javadoc
 *
 * Revision 1.3  2003/01/09 16:50:42  gus
 * Merging all static utilities into one class. Util.java is now just a shell
 *
 * Revision 1.2  2003/01/09 16:34:32  gus
 *
 * Unused constant removed:
 * ------------------------
 * CUR
 *
 * Unused Methods removed:
 * ------------------------
 * public static void delFile(String)
 * public static void copyFiles(String, String)
 *
 */